@@ -2,16 +2,16 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** Commercial Usage
+** No Commercial Usage
 **
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** This file contains pre-release code and may not be distributed.
+** You may use this file in accordance with the terms and conditions
+** contained in the Technology Preview License Agreement accompanying
+** this package.
 **
 ** GNU Lesser General Public License Usage
 **
@@ -22,21 +22,25 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** If you are unsure which license is appropriate for your use, please
-** contact the sales department at http://qt.nokia.com/contact.
+** In addition, as a special exception, Nokia gives you certain additional
+** rights.  These rights are described in the Nokia Qt LGPL Exception
+** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+**
+** If you have questions regarding the use of this file, please contact
+** Nokia at qt-info@nokia.com.
 **
 **************************************************************************/
 
 #include "environmenteditmodel.h"
 
 #include <utils/detailswidget.h>
-
+#include <utils/environment.h>
 
 #include <QtGui/QTextDocument>
 #include <QtGui/QVBoxLayout>
 #include <QtGui/QHeaderView>
 #include <QtGui/QToolButton>
-#include <QtCore/QDebug>
+
 #include <QtGui/QWidget>
 #include <QtGui/QCheckBox>
 #include <QtGui/QTableView>
@@ -44,13 +48,55 @@
 #include <QtGui/QLabel>
 #include <QtGui/QStackedWidget>
 
-using namespace ProjectExplorer;
+#include <QtCore/QString>
+#include <QtCore/QAbstractTableModel>
+#include <QtCore/QDebug>
 
-EnvironmentModel::EnvironmentModel()
-{}
+namespace ProjectExplorer {
 
-EnvironmentModel::~EnvironmentModel()
-{}
+class EnvironmentModel : public QAbstractTableModel
+{
+    Q_OBJECT
+public:
+    EnvironmentModel() {}
+
+    int rowCount(const QModelIndex &parent) const;
+    int columnCount(const QModelIndex &parent) const;
+    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
+    bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole);
+    Qt::ItemFlags flags(const QModelIndex &index) const;
+    QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
+
+    QModelIndex addVariable();
+    QModelIndex addVariable(const Utils::EnvironmentItem &item);
+    void resetVariable(const QString &name);
+    void unsetVariable(const QString &name);
+    bool canUnset(const QString &name);
+    bool canReset(const QString &name);
+    QString indexToVariable(const QModelIndex &index) const;
+    QModelIndex variableToIndex(const QString &name) const;
+    bool changes(const QString &key) const;
+    void setBaseEnvironment(const Utils::Environment &env);
+    QList<Utils::EnvironmentItem> userChanges() const;
+    void setUserChanges(QList<Utils::EnvironmentItem> list);
+
+signals:
+    void userChangesChanged();
+    /// Hint to the view where it should make sense to focus on next
+    // This is a hack since there is no way for a model to suggest
+    // the next interesting place to focus on to the view.
+    void focusIndex(const QModelIndex &index);
+
+private:
+    void updateResultEnvironment();
+    int findInChanges(const QString &name) const;
+    int findInResultInsertPosition(const QString &name) const;
+    int findInResult(const QString &name) const;
+
+    Utils::Environment m_baseEnvironment;
+    Utils::Environment m_resultEnvironment;
+    QList<Utils::EnvironmentItem> m_items;
+};
 
 QString EnvironmentModel::indexToVariable(const QModelIndex &index) const
 {
@@ -63,14 +109,14 @@ void EnvironmentModel::updateResultEnvironment()
     m_resultEnvironment.modify(m_items);
     // Add removed variables again and mark them as "<UNSET>" so
     // that the user can actually see those removals:
-    foreach (const EnvironmentItem &item, m_items) {
+    foreach (const Utils::EnvironmentItem &item, m_items) {
         if (item.unset) {
             m_resultEnvironment.set(item.name, tr("<UNSET>"));
         }
     }
 }
 
-void EnvironmentModel::setBaseEnvironment(const ProjectExplorer::Environment &env)
+void EnvironmentModel::setBaseEnvironment(const Utils::Environment &env)
 {
     if (m_baseEnvironment == env)
         return;
@@ -164,7 +210,7 @@ QModelIndex EnvironmentModel::variableToIndex(const QString &name) const
 
 int EnvironmentModel::findInResult(const QString &name) const
 {
-    Environment::const_iterator it;
+    Utils::Environment::const_iterator it;
     int i = 0;
     for (it = m_resultEnvironment.constBegin(); it != m_resultEnvironment.constEnd(); ++it, ++i)
         if (m_resultEnvironment.key(it) == name)
@@ -174,7 +220,7 @@ int EnvironmentModel::findInResult(const QString &name) const
 
 int EnvironmentModel::findInResultInsertPosition(const QString &name) const
 {
-    Environment::const_iterator it;
+    Utils::Environment::const_iterator it;
     int i = 0;
     for (it = m_resultEnvironment.constBegin(); it != m_resultEnvironment.constEnd(); ++it, ++i)
         if (m_resultEnvironment.key(it) > name)
@@ -206,7 +252,7 @@ bool EnvironmentModel::setData(const QModelIndex &index, const QVariant &value,
         if (m_resultEnvironment.hasKey(newName))
             return false;
 
-        EnvironmentItem newVariable(newName, oldValue);
+        Utils::EnvironmentItem newVariable(newName, oldValue);
 
         if (changesPos != -1)
             resetVariable(oldName); // restore the original base variable again
@@ -229,7 +275,7 @@ bool EnvironmentModel::setData(const QModelIndex &index, const QVariant &value,
             }
         } else {
             // Add a new change item:
-            m_items.append(EnvironmentItem(oldName, stringValue));
+            m_items.append(Utils::EnvironmentItem(oldName, stringValue));
         }
         updateResultEnvironment();
         emit dataChanged(index, index);
@@ -241,13 +287,13 @@ bool EnvironmentModel::setData(const QModelIndex &index, const QVariant &value,
 
 QModelIndex EnvironmentModel::addVariable()
 {
-    return addVariable(EnvironmentItem(tr("<VARIABLE>",
-                                          "Name when inserting a new variable"),
-                                       tr("<VALUE>",
-                                          "Value when inserting a new variable")));
+    //: Name when inserting a new variable
+    return addVariable(Utils::EnvironmentItem(tr("<VARIABLE>"),
+                                              //: Value when inserting a new variable
+                                              tr("<VALUE>")));
 }
 
-QModelIndex EnvironmentModel::addVariable(const EnvironmentItem &item)
+QModelIndex EnvironmentModel::addVariable(const Utils::EnvironmentItem &item)
 {
     int insertPos = findInResultInsertPosition(item.name);
 
@@ -323,7 +369,7 @@ void EnvironmentModel::unsetVariable(const QString &name)
         emit userChangesChanged();
         return;
     }
-    EnvironmentItem item(name, QString());
+    Utils::EnvironmentItem item(name, QString());
     item.unset = true;
     m_items.append(item);
     updateResultEnvironment();
@@ -345,12 +391,12 @@ bool EnvironmentModel::canReset(const QString &name)
     return m_baseEnvironment.hasKey(name);
 }
 
-QList<EnvironmentItem> EnvironmentModel::userChanges() const
+QList<Utils::EnvironmentItem> EnvironmentModel::userChanges() const
 {
     return m_items;
 }
 
-void EnvironmentModel::setUserChanges(QList<EnvironmentItem> list)
+void EnvironmentModel::setUserChanges(QList<Utils::EnvironmentItem> list)
 {
     // We assume nobody is reordering the items here.
     if (list == m_items)
@@ -365,25 +411,37 @@ void EnvironmentModel::setUserChanges(QList<EnvironmentItem> list)
 // EnvironmentWidget::EnvironmentWidget
 ////
 
+struct EnvironmentWidgetPrivate {
+    EnvironmentModel *m_model;
+
+    QString m_baseEnvironmentText;
+    Utils::DetailsWidget *m_detailsContainer;
+    QTableView *m_environmentView;
+    QPushButton *m_editButton;
+    QPushButton *m_addButton;
+    QPushButton *m_resetButton;
+    QPushButton *m_unsetButton;
+};
+
 EnvironmentWidget::EnvironmentWidget(QWidget *parent, QWidget *additionalDetailsWidget)
-    : QWidget(parent)
+    : QWidget(parent), d(new EnvironmentWidgetPrivate)
 {
-    m_model = new EnvironmentModel();
-    connect(m_model, SIGNAL(userChangesChanged()),
+    d->m_model = new EnvironmentModel();
+    connect(d->m_model, SIGNAL(userChangesChanged()),
             this, SIGNAL(userChangesChanged()));
-    connect(m_model, SIGNAL(modelReset()),
+    connect(d->m_model, SIGNAL(modelReset()),
             this, SLOT(invalidateCurrentIndex()));
 
-    connect(m_model, SIGNAL(focusIndex(QModelIndex)),
+    connect(d->m_model, SIGNAL(focusIndex(QModelIndex)),
             this, SLOT(focusIndex(QModelIndex)));
 
     QVBoxLayout *vbox = new QVBoxLayout(this);
     vbox->setContentsMargins(0, 0, 0, 0);
 
-    m_detailsContainer = new Utils::DetailsWidget(this);
+    d->m_detailsContainer = new Utils::DetailsWidget(this);
 
-    QWidget *details = new QWidget(m_detailsContainer);
-    m_detailsContainer->setWidget(details);
+    QWidget *details = new QWidget(d->m_detailsContainer);
+    d->m_detailsContainer->setWidget(details);
     details->setVisible(false);
 
     QVBoxLayout *vbox2 = new QVBoxLayout(details);
@@ -394,102 +452,102 @@ EnvironmentWidget::EnvironmentWidget(QWidget *parent, QWidget *additionalDetails
 
     QHBoxLayout *horizontalLayout = new QHBoxLayout();
     horizontalLayout->setMargin(0);
-    m_environmentView = new QTableView(this);
-    m_environmentView->setModel(m_model);
-    m_environmentView->setMinimumHeight(400);
-    m_environmentView->setGridStyle(Qt::NoPen);
-    m_environmentView->horizontalHeader()->setStretchLastSection(true);
-    m_environmentView->horizontalHeader()->setResizeMode(0, QHeaderView::ResizeToContents);
-    m_environmentView->horizontalHeader()->setHighlightSections(false);
-    m_environmentView->verticalHeader()->hide();
+    d->m_environmentView = new QTableView(this);
+    d->m_environmentView->setModel(d->m_model);
+    d->m_environmentView->setMinimumHeight(400);
+    d->m_environmentView->setGridStyle(Qt::NoPen);
+    d->m_environmentView->horizontalHeader()->setStretchLastSection(true);
+    d->m_environmentView->horizontalHeader()->setResizeMode(0, QHeaderView::ResizeToContents);
+    d->m_environmentView->horizontalHeader()->setHighlightSections(false);
+    d->m_environmentView->verticalHeader()->hide();
     QFontMetrics fm(font());
-    m_environmentView->verticalHeader()->setDefaultSectionSize(qMax(static_cast<int>(fm.height() * 1.2), fm.height() + 4));
-    m_environmentView->setSelectionMode(QAbstractItemView::SingleSelection);
-    horizontalLayout->addWidget(m_environmentView);
+    d->m_environmentView->verticalHeader()->setDefaultSectionSize(qMax(static_cast<int>(fm.height() * 1.2), fm.height() + 4));
+    d->m_environmentView->setSelectionMode(QAbstractItemView::SingleSelection);
+    horizontalLayout->addWidget(d->m_environmentView);
 
     QVBoxLayout *buttonLayout = new QVBoxLayout();
 
-    m_editButton = new QPushButton(this);
-    m_editButton->setText(tr("&Edit"));
-    buttonLayout->addWidget(m_editButton);
+    d->m_editButton = new QPushButton(this);
+    d->m_editButton->setText(tr("&Edit"));
+    buttonLayout->addWidget(d->m_editButton);
 
-    m_addButton = new QPushButton(this);
-    m_addButton->setText(tr("&Add"));
-    buttonLayout->addWidget(m_addButton);
+    d->m_addButton = new QPushButton(this);
+    d->m_addButton->setText(tr("&Add"));
+    buttonLayout->addWidget(d->m_addButton);
 
-    m_resetButton = new QPushButton(this);
-    m_resetButton->setEnabled(false);
-    m_resetButton->setText(tr("&Reset"));
-    buttonLayout->addWidget(m_resetButton);
+    d->m_resetButton = new QPushButton(this);
+    d->m_resetButton->setEnabled(false);
+    d->m_resetButton->setText(tr("&Reset"));
+    buttonLayout->addWidget(d->m_resetButton);
 
-    m_unsetButton = new QPushButton(this);
-    m_unsetButton->setEnabled(false);
-    m_unsetButton->setText(tr("&Unset"));
-    buttonLayout->addWidget(m_unsetButton);
+    d->m_unsetButton = new QPushButton(this);
+    d->m_unsetButton->setEnabled(false);
+    d->m_unsetButton->setText(tr("&Unset"));
+    buttonLayout->addWidget(d->m_unsetButton);
 
     QSpacerItem *verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);
     buttonLayout->addItem(verticalSpacer);
     horizontalLayout->addLayout(buttonLayout);
     vbox2->addLayout(horizontalLayout);
 
-    vbox->addWidget(m_detailsContainer);
+    vbox->addWidget(d->m_detailsContainer);
 
-    connect(m_model, SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)),
+    connect(d->m_model, SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)),
             this, SLOT(updateButtons()));
 
-    connect(m_editButton, SIGNAL(clicked(bool)),
+    connect(d->m_editButton, SIGNAL(clicked(bool)),
             this, SLOT(editEnvironmentButtonClicked()));
-    connect(m_addButton, SIGNAL(clicked(bool)),
+    connect(d->m_addButton, SIGNAL(clicked(bool)),
             this, SLOT(addEnvironmentButtonClicked()));
-    connect(m_resetButton, SIGNAL(clicked(bool)),
+    connect(d->m_resetButton, SIGNAL(clicked(bool)),
             this, SLOT(removeEnvironmentButtonClicked()));
-    connect(m_unsetButton, SIGNAL(clicked(bool)),
+    connect(d->m_unsetButton, SIGNAL(clicked(bool)),
             this, SLOT(unsetEnvironmentButtonClicked()));
-    connect(m_environmentView->selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)),
+    connect(d->m_environmentView->selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)),
             this, SLOT(environmentCurrentIndexChanged(QModelIndex)));
 
-    connect(m_model, SIGNAL(userChangesChanged()), this, SLOT(updateSummaryText()));
+    connect(d->m_model, SIGNAL(userChangesChanged()), this, SLOT(updateSummaryText()));
 }
 
 EnvironmentWidget::~EnvironmentWidget()
 {
-    delete m_model;
-    m_model = 0;
+    delete d->m_model;
+    d->m_model = 0;
 }
 
 void EnvironmentWidget::focusIndex(const QModelIndex &index)
 {
-    m_environmentView->setCurrentIndex(index);
-    m_environmentView->setFocus();
+    d->m_environmentView->setCurrentIndex(index);
+    d->m_environmentView->setFocus();
 }
 
-void EnvironmentWidget::setBaseEnvironment(const ProjectExplorer::Environment &env)
+void EnvironmentWidget::setBaseEnvironment(const Utils::Environment &env)
 {
-    m_model->setBaseEnvironment(env);
+    d->m_model->setBaseEnvironment(env);
 }
 
 void EnvironmentWidget::setBaseEnvironmentText(const QString &text)
 {
-    m_baseEnvironmentText = text;
+    d->m_baseEnvironmentText = text;
     updateSummaryText();
 }
 
-QList<EnvironmentItem> EnvironmentWidget::userChanges() const
+QList<Utils::EnvironmentItem> EnvironmentWidget::userChanges() const
 {
-    return m_model->userChanges();
+    return d->m_model->userChanges();
 }
 
-void EnvironmentWidget::setUserChanges(QList<EnvironmentItem> list)
+void EnvironmentWidget::setUserChanges(const QList<Utils::EnvironmentItem> &list)
 {
-    m_model->setUserChanges(list);
+    d->m_model->setUserChanges(list);
     updateSummaryText();
 }
 
 void EnvironmentWidget::updateSummaryText()
 {
     QString text;
-    const QList<EnvironmentItem> &list = m_model->userChanges();
-    foreach (const EnvironmentItem &item, list) {
+    const QList<Utils::EnvironmentItem> &list = d->m_model->userChanges();
+    foreach (const Utils::EnvironmentItem &item, list) {
         if (item.name != EnvironmentModel::tr("<VARIABLE>")) {
             text.append("<br>");
             if (item.unset)
@@ -500,60 +558,60 @@ void EnvironmentWidget::updateSummaryText()
     }
 
     if (text.isEmpty())
-        text.prepend(tr("Using <b>%1</b>").arg(m_baseEnvironmentText));
+        text.prepend(tr("Using <b>%1</b>").arg(d->m_baseEnvironmentText));
     else
-        text.prepend(tr("Using <b>%1</b> and").arg(m_baseEnvironmentText));
+        text.prepend(tr("Using <b>%1</b> and").arg(d->m_baseEnvironmentText));
 
-    m_detailsContainer->setSummaryText(text);
+    d->m_detailsContainer->setSummaryText(text);
 }
 
 void EnvironmentWidget::updateButtons()
 {
-    environmentCurrentIndexChanged(m_environmentView->currentIndex());
+    environmentCurrentIndexChanged(d->m_environmentView->currentIndex());
 }
 
 void EnvironmentWidget::editEnvironmentButtonClicked()
 {
-    m_environmentView->edit(m_environmentView->currentIndex());
+    d->m_environmentView->edit(d->m_environmentView->currentIndex());
 }
 
 void EnvironmentWidget::addEnvironmentButtonClicked()
 {
-    QModelIndex index = m_model->addVariable();
-    m_environmentView->setCurrentIndex(index);
-    m_environmentView->edit(index);
+    QModelIndex index = d->m_model->addVariable();
+    d->m_environmentView->setCurrentIndex(index);
+    d->m_environmentView->edit(index);
 }
 
 void EnvironmentWidget::removeEnvironmentButtonClicked()
 {
-    const QString &name = m_model->indexToVariable(m_environmentView->currentIndex());
-    m_model->resetVariable(name);
+    const QString &name = d->m_model->indexToVariable(d->m_environmentView->currentIndex());
+    d->m_model->resetVariable(name);
 }
 
 // unset in Merged Environment Mode means, unset if it comes from the base environment
 // or remove when it is just a change we added
 void EnvironmentWidget::unsetEnvironmentButtonClicked()
 {
-    const QString &name = m_model->indexToVariable(m_environmentView->currentIndex());
-    if (!m_model->canReset(name))
-        m_model->resetVariable(name);
+    const QString &name = d->m_model->indexToVariable(d->m_environmentView->currentIndex());
+    if (!d->m_model->canReset(name))
+        d->m_model->resetVariable(name);
     else
-        m_model->unsetVariable(name);
+        d->m_model->unsetVariable(name);
 }
 
 void EnvironmentWidget::environmentCurrentIndexChanged(const QModelIndex &current)
 {
     if (current.isValid()) {
-        m_editButton->setEnabled(true);
-        const QString &name = m_model->indexToVariable(current);
-        bool modified = m_model->canReset(name) && m_model->changes(name);
-        bool unset = m_model->canUnset(name);
-        m_resetButton->setEnabled(modified || unset);
-        m_unsetButton->setEnabled(!unset);
+        d->m_editButton->setEnabled(true);
+        const QString &name = d->m_model->indexToVariable(current);
+        bool modified = d->m_model->canReset(name) && d->m_model->changes(name);
+        bool unset = d->m_model->canUnset(name);
+        d->m_resetButton->setEnabled(modified || unset);
+        d->m_unsetButton->setEnabled(!unset);
     } else {
-        m_editButton->setEnabled(false);
-        m_resetButton->setEnabled(false);
-        m_unsetButton->setEnabled(false);
+        d->m_editButton->setEnabled(false);
+        d->m_resetButton->setEnabled(false);
+        d->m_unsetButton->setEnabled(false);
     }
 }
 
@@ -561,3 +619,7 @@ void EnvironmentWidget::invalidateCurrentIndex()
 {
     environmentCurrentIndexChanged(QModelIndex());
 }
+
+} // namespace ProjectExplorer
+
+#include "environmenteditmodel.moc"
