@@ -2,16 +2,16 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** Commercial Usage
+** No Commercial Usage
 **
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** This file contains pre-release code and may not be distributed.
+** You may use this file in accordance with the terms and conditions
+** contained in the Technology Preview License Agreement accompanying
+** this package.
 **
 ** GNU Lesser General Public License Usage
 **
@@ -22,8 +22,12 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** If you are unsure which license is appropriate for your use, please
-** contact the sales department at http://qt.nokia.com/contact.
+** In addition, as a special exception, Nokia gives you certain additional
+** rights.  These rights are described in the Nokia Qt LGPL Exception
+** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+**
+** If you have questions regarding the use of this file, please contact
+** Nokia at qt-info@nokia.com.
 **
 **************************************************************************/
 
@@ -49,7 +53,6 @@
 #include <coreplugin/filemanager.h>
 #include <coreplugin/messagemanager.h>
 #include <coreplugin/mimedatabase.h>
-#include <coreplugin/uniqueidmanager.h>
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/actionmanager/command.h>
@@ -66,13 +69,14 @@
 #include <QtCore/QTextCodec>
 #include <QtCore/QtPlugin>
 #include <QtCore/QProcessEnvironment>
+#include <QtCore/QUrl>
 #include <QtGui/QAction>
 #include <QtGui/QFileDialog>
 #include <QtGui/QMainWindow>
 #include <QtGui/QMenu>
 #include <QtGui/QMessageBox>
 #include <QtGui/QInputDialog>
-
+#include <QtXml/QXmlStreamReader>
 #include <limits.h>
 
 using namespace Subversion::Internal;
@@ -105,6 +109,8 @@ static const char * const CMD_ID_DESCRIBE           = "Subversion.Describe";
 
 static const char *nonInteractiveOptionC = "--non-interactive";
 
+
+
 static const VCSBase::VCSBaseEditorParameters editorParameters[] = {
 {
     VCSBase::RegularCommandOutput,
@@ -250,7 +256,7 @@ static const VCSBase::VCSBaseSubmitEditorParameters submitParameters = {
 static inline Core::Command *createSeparator(QObject *parent,
                                              Core::ActionManager *ami,
                                              const char*id,
-                                             const QList<int> &globalcontext)
+                                             const Core::Context &globalcontext)
 {
     QAction *tmpaction = new QAction(parent);
     tmpaction->setSeparator(true);
@@ -298,13 +304,11 @@ bool SubversionPlugin::initialize(const QStringList & /*arguments */, QString *e
     Core::ActionContainer *toolsContainer = ami->actionContainer(M_TOOLS);
 
     Core::ActionContainer *subversionMenu =
-        ami->createMenu(QLatin1String(CMD_ID_SUBVERSION_MENU));
+        ami->createMenu(Core::Id(CMD_ID_SUBVERSION_MENU));
     subversionMenu->menu()->setTitle(tr("&Subversion"));
     toolsContainer->addMenu(subversionMenu);
     m_menuAction = subversionMenu->menu()->menuAction();
-    QList<int> globalcontext;
-    globalcontext << core->uniqueIDManager()->uniqueIdentifier(C_GLOBAL);
-
+    Core::Context globalcontext(C_GLOBAL);
     Core::Command *command;
 
     m_diffCurrentAction = new Utils::ParameterAction(tr("Diff Current File"), tr("Diff \"%1\""), Utils::ParameterAction::EnabledWithParameter, this);
@@ -455,11 +459,11 @@ bool SubversionPlugin::initialize(const QStringList & /*arguments */, QString *e
     m_commandLocator->appendCommand(command);
 
     // Actions of the submit editor
-    QList<int> svncommitcontext;
-    svncommitcontext << Core::UniqueIDManager::instance()->uniqueIdentifier(Constants::SUBVERSIONCOMMITEDITOR);
+    Core::Context svncommitcontext(Constants::SUBVERSIONCOMMITEDITOR);
 
     m_submitCurrentLogAction = new QAction(VCSBase::VCSBaseSubmitEditor::submitIcon(), tr("Commit"), this);
     command = ami->registerAction(m_submitCurrentLogAction, Constants::SUBMIT_CURRENT, svncommitcontext);
+    command->setAttribute(Core::Command::CA_UpdateText);
     connect(m_submitCurrentLogAction, SIGNAL(triggered()), this, SLOT(submitCurrentLog()));
 
     m_submitDiffAction = new QAction(VCSBase::VCSBaseSubmitEditor::diffIcon(), tr("Diff Selected Files"), this);
@@ -548,7 +552,8 @@ void SubversionPlugin::svnDiff(const QString &workingDir, const QStringList &fil
     QStringList args(QLatin1String("diff"));
     args << files;
 
-    const SubversionResponse response = runSvn(workingDir, args, m_settings.timeOutMS(), false, codec);
+    const SubversionResponse response =
+            runSvn(workingDir, args, m_settings.timeOutMS(), 0);
     if (response.error)
         return;
 
@@ -558,7 +563,7 @@ void SubversionPlugin::svnDiff(const QString &workingDir, const QStringList &fil
         // Show in the same editor if diff has been executed before
         if (Core::IEditor *editor = locateEditor("originalFileName", files.front())) {
             editor->createNew(response.stdOut);
-            Core::EditorManager::instance()->activateEditor(editor);
+            Core::EditorManager::instance()->activateEditor(editor, Core::EditorManager::ModeSwitch);
             setDiffBaseDirectory(editor, workingDir);
             return;
         }
@@ -572,7 +577,9 @@ void SubversionPlugin::svnDiff(const QString &workingDir, const QStringList &fil
 
 SubversionSubmitEditor *SubversionPlugin::openSubversionSubmitEditor(const QString &fileName)
 {
-    Core::IEditor *editor = Core::EditorManager::instance()->openEditor(fileName, QLatin1String(Constants::SUBVERSIONCOMMITEDITOR_ID));
+    Core::IEditor *editor = Core::EditorManager::instance()->openEditor(fileName,
+                                                                        QLatin1String(Constants::SUBVERSIONCOMMITEDITOR_ID),
+                                                                        Core::EditorManager::ModeSwitch);
     SubversionSubmitEditor *submitEditor = qobject_cast<SubversionSubmitEditor*>(editor);
     QTC_ASSERT(submitEditor, /**/);
     submitEditor->registerActions(m_submitUndoAction, m_submitRedoAction, m_submitCurrentLogAction, m_submitDiffAction);
@@ -629,13 +636,15 @@ void SubversionPlugin::revertAll()
     const VCSBase::VCSBasePluginState state = currentState();
     QTC_ASSERT(state.hasTopLevel(), return)
     const QString title = tr("Revert repository");
-    if (QMessageBox::warning(0, title, tr("Would you like to revert all changes to the repository?"),
+    if (QMessageBox::warning(0, title, tr("Revert all pending changes to the repository?"),
                              QMessageBox::Yes, QMessageBox::No) == QMessageBox::No)
         return;
     // NoteL: Svn "revert ." doesn not work.
     QStringList args;
     args << QLatin1String("revert") << QLatin1String("--recursive") << state.topLevel();
-    const SubversionResponse revertResponse = runSvn(state.topLevel(), args, m_settings.timeOutMS(), true);
+    const SubversionResponse revertResponse =
+            runSvn(state.topLevel(), args, m_settings.timeOutMS(),
+                   SshPasswordPrompt|ShowStdOutInLogWindow);
     if (revertResponse.error) {
         QMessageBox::warning(0, title, tr("Revert failed: %1").arg(revertResponse.message), QMessageBox::Ok);
     } else {
@@ -651,7 +660,8 @@ void SubversionPlugin::revertCurrentFile()
     QStringList args(QLatin1String("diff"));
     args.push_back(state.relativeCurrentFile());
 
-    const SubversionResponse diffResponse = runSvn(state.currentFileTopLevel(), args, m_settings.timeOutMS(), false);
+    const SubversionResponse diffResponse =
+            runSvn(state.currentFileTopLevel(), args, m_settings.timeOutMS(), 0);
     if (diffResponse.error)
         return;
 
@@ -668,7 +678,10 @@ void SubversionPlugin::revertCurrentFile()
     args.clear();
     args << QLatin1String("revert") << state.relativeCurrentFile();
 
-    const SubversionResponse revertResponse = runSvn(state.currentFileTopLevel(), args, m_settings.timeOutMS(), true);
+    const SubversionResponse revertResponse =
+            runSvn(state.currentFileTopLevel(), args, m_settings.timeOutMS(),
+                   SshPasswordPrompt|ShowStdOutInLogWindow);
+
     if (!revertResponse.error) {
         subVersionControl()->emitFilesChanged(QStringList(state.currentFile()));
     }
@@ -724,7 +737,8 @@ void SubversionPlugin::startCommit(const QString &workingDir, const QStringList
     QStringList args(QLatin1String("status"));
     args += files;
 
-    const SubversionResponse response = runSvn(workingDir, args, m_settings.timeOutMS(), false);
+    const SubversionResponse response =
+            runSvn(workingDir, args, m_settings.timeOutMS(), 0);
     if (response.error)
         return;
 
@@ -765,7 +779,9 @@ bool SubversionPlugin::commit(const QString &messageFile,
     QStringList args = QStringList(QLatin1String("commit"));
     args << QLatin1String(nonInteractiveOptionC) << QLatin1String("--file") << messageFile;
     args.append(subVersionFileList);
-    const SubversionResponse response = runSvn(m_commitRepository, args, m_settings.longTimeOutMS(), true);
+    const SubversionResponse response =
+            runSvn(m_commitRepository, args, m_settings.longTimeOutMS(),
+                   SshPasswordPrompt|ShowStdOutInLogWindow);
     return !response.error ;
 }
 
@@ -820,7 +836,8 @@ void SubversionPlugin::svnStatus(const QString &workingDir, const QStringList &r
         args.append(relativePaths);
     VCSBase::VCSBaseOutputWindow *outwin = VCSBase::VCSBaseOutputWindow::instance();
     outwin->setRepository(workingDir);
-    runSvn(workingDir, args, m_settings.timeOutMS(), true);
+    runSvn(workingDir, args, m_settings.timeOutMS(),
+           ShowStdOutInLogWindow|ShowSuccessMessage);
     outwin->clearRepository();
 }
 
@@ -836,7 +853,9 @@ void SubversionPlugin::filelog(const QString &workingDir,
     foreach(const QString &file, files)
         args.append(QDir::toNativeSeparators(file));
 
-    const SubversionResponse response = runSvn(workingDir, args, m_settings.timeOutMS(), false, codec);
+    const SubversionResponse response =
+            runSvn(workingDir, args, m_settings.timeOutMS(),
+                   SshPasswordPrompt, codec);
     if (response.error)
         return;
 
@@ -846,7 +865,7 @@ void SubversionPlugin::filelog(const QString &workingDir,
     const QString id = VCSBase::VCSBaseEditor::getTitleId(workingDir, files);
     if (Core::IEditor *editor = locateEditor("logFileName", id)) {
         editor->createNew(response.stdOut);
-        Core::EditorManager::instance()->activateEditor(editor);
+        Core::EditorManager::instance()->activateEditor(editor, Core::EditorManager::ModeSwitch);
     } else {
         const QString title = QString::fromLatin1("svn log %1").arg(id);
         const QString source = VCSBase::VCSBaseEditor::getSource(workingDir, files);
@@ -870,7 +889,9 @@ void SubversionPlugin::svnUpdate(const QString &workingDir, const QStringList &r
     args.push_back(QLatin1String(nonInteractiveOptionC));
     if (!relativePaths.isEmpty())
         args.append(relativePaths);
-        const SubversionResponse response = runSvn(workingDir, args, m_settings.longTimeOutMS(), true);
+        const SubversionResponse response =
+                runSvn(workingDir, args, m_settings.longTimeOutMS(),
+                       SshPasswordPrompt|ShowStdOutInLogWindow);
     if (!response.error)
         subVersionControl()->emitRepositoryChanged(workingDir);
 }
@@ -904,7 +925,9 @@ void SubversionPlugin::vcsAnnotate(const QString &workingDir, const QString &fil
     args.push_back(QLatin1String("-v"));
     args.append(QDir::toNativeSeparators(file));
 
-    const SubversionResponse response = runSvn(workingDir, args, m_settings.timeOutMS(), false, codec);
+    const SubversionResponse response =
+            runSvn(workingDir, args, m_settings.timeOutMS(),
+                   SshPasswordPrompt, codec);
     if (response.error)
         return;
 
@@ -919,7 +942,7 @@ void SubversionPlugin::vcsAnnotate(const QString &workingDir, const QString &fil
     if (Core::IEditor *editor = locateEditor("annotateFileName", id)) {
         editor->createNew(response.stdOut);
         VCSBase::VCSBaseEditor::gotoLineOfEditor(editor, lineNumber);
-        Core::EditorManager::instance()->activateEditor(editor);
+        Core::EditorManager::instance()->activateEditor(editor, Core::EditorManager::ModeSwitch);
     } else {
         const QString title = QString::fromLatin1("svn annotate %1").arg(id);
         Core::IEditor *newEditor = showOutputInEditor(title, response.stdOut, VCSBase::AnnotateOutput, source, codec);
@@ -940,8 +963,9 @@ void SubversionPlugin::describe(const QString &source, const QString &changeNr)
     // To describe a complete change, find the top level and then do
     //svn diff -r 472958:472959 <top level>
     const QFileInfo fi(source);
-    const QString topLevel = findTopLevelForDirectory(fi.isDir() ? source : fi.absolutePath());
-    if (topLevel.isEmpty())
+    QString topLevel;
+    const bool manages = managesDirectory(fi.isDir() ? source : fi.absolutePath(), &topLevel);
+    if (!manages || topLevel.isEmpty())
         return;
     if (Subversion::Constants::debug)
         qDebug() << Q_FUNC_INFO << source << topLevel << changeNr;
@@ -955,7 +979,8 @@ void SubversionPlugin::describe(const QString &source, const QString &changeNr)
     QStringList args(QLatin1String("log"));
     args.push_back(QLatin1String("-r"));
     args.push_back(changeNr);
-    const SubversionResponse logResponse = runSvn(topLevel, args, m_settings.timeOutMS(), false);
+    const SubversionResponse logResponse =
+            runSvn(topLevel, args, m_settings.timeOutMS(), SshPasswordPrompt);
     if (logResponse.error)
         return;
     description = logResponse.stdOut;
@@ -969,7 +994,9 @@ void SubversionPlugin::describe(const QString &source, const QString &changeNr)
     args.push_back(diffArg);
 
     QTextCodec *codec = VCSBase::VCSBaseEditor::getCodec(source);
-    const SubversionResponse response = runSvn(topLevel, args, m_settings.timeOutMS(), false, codec);
+    const SubversionResponse response =
+            runSvn(topLevel, args, m_settings.timeOutMS(),
+                   SshPasswordPrompt, codec);
     if (response.error)
         return;
     description += response.stdOut;
@@ -979,7 +1006,7 @@ void SubversionPlugin::describe(const QString &source, const QString &changeNr)
     const QString id = diffArg + source;
     if (Core::IEditor *editor = locateEditor("describeChange", id)) {
         editor->createNew(description);
-        Core::EditorManager::instance()->activateEditor(editor);
+        Core::EditorManager::instance()->activateEditor(editor, Core::EditorManager::ModeSwitch);
     } else {
         const QString title = QString::fromLatin1("svn describe %1#%2").arg(fi.fileName(), changeNr);
         Core::IEditor *newEditor = showOutputInEditor(title, description, VCSBase::DiffOutput, source, codec);
@@ -1012,33 +1039,46 @@ void SubversionPlugin::submitCurrentLog()
         << Core::EditorManager::instance()->currentEditor());
 }
 
-static inline QString processStdErr(QProcess &proc)
+SubversionResponse
+        SubversionPlugin::runSvn(const QString &workingDir,
+                                 const QStringList &arguments,
+                                 int timeOut,
+                                 unsigned flags,
+                                 QTextCodec *outputCodec)
 {
-    return QString::fromLocal8Bit(proc.readAllStandardError()).remove(QLatin1Char('\r'));
+
+    return m_settings.hasAuthentication() ?
+            runSvn(workingDir, m_settings.user, m_settings.password,
+                   arguments, timeOut, flags, outputCodec) :
+            runSvn(workingDir, QString(), QString(),
+                   arguments, timeOut, flags, outputCodec);
 }
 
-static inline QString processStdOut(QProcess &proc, QTextCodec *outputCodec = 0)
+// Add authorization options to the command line arguments.
+// SVN pre 1.5 does not accept "--userName" for "add", which is most likely
+// an oversight. As no password is needed for the option, generally omit it.
+QStringList SubversionPlugin::addAuthenticationOptions(const QStringList &args,
+                                                      const QString &userName, const QString &password)
 {
-    const QByteArray stdOutData = proc.readAllStandardOutput();
-    QString stdOut = outputCodec ? outputCodec->toUnicode(stdOutData) : QString::fromLocal8Bit(stdOutData);
-    return stdOut.remove(QLatin1Char('\r'));
-}
-
-// Format log entry for command
-static inline QString msgExecutionLogEntry(const QString &workingDir, const QString &executable, const QStringList &arguments)
-{
-    const QString argsS = SubversionSettings::formatArguments(arguments);
-    if (workingDir.isEmpty())
-        return SubversionPlugin::tr("Executing: %1 %2\n").arg(executable, argsS);
-    return SubversionPlugin::tr("Executing in %1: %2 %3\n").
-            arg(QDir::toNativeSeparators(workingDir), executable, argsS);
+    if (userName.isEmpty())
+        return args;
+    if (!args.empty() && args.front() == QLatin1String("add"))
+        return args;
+    QStringList rc;
+    rc.push_back(QLatin1String("--username"));
+    rc.push_back(userName);
+    if (!password.isEmpty()) {
+        rc.push_back(QLatin1String("--password"));
+        rc.push_back(password);
+    }
+    rc.append(args);
+    return rc;
 }
 
 SubversionResponse SubversionPlugin::runSvn(const QString &workingDir,
-                                            const QStringList &arguments,
-                                            int timeOut,
-                                            bool showStdOutInOutputWindow,
-                                            QTextCodec *outputCodec)
+                          const QString &userName, const QString &password,
+                          const QStringList &arguments, int timeOut,
+                          unsigned flags, QTextCodec *outputCodec)
 {
     const QString executable = m_settings.svnCommand;
     SubversionResponse response;
@@ -1047,57 +1087,17 @@ SubversionResponse SubversionPlugin::runSvn(const QString &workingDir,
         response.message =tr("No subversion executable specified!");
         return response;
     }
-    const QStringList allArgs = m_settings.addOptions(arguments);
 
-    VCSBase::VCSBaseOutputWindow *outputWindow = VCSBase::VCSBaseOutputWindow::instance();
-    // Hide passwords, etc in the log window
-    //: Executing: <executable> <arguments>
-    const QString outputText = msgExecutionLogEntry(workingDir, executable, allArgs);
-    outputWindow->appendCommand(outputText);
+    const QStringList completeArguments = SubversionPlugin::addAuthenticationOptions(arguments, userName, password);
+    const Utils::SynchronousProcessResponse sp_resp =
+            VCSBase::VCSBasePlugin::runVCS(workingDir, executable,
+                                           completeArguments, timeOut, flags, outputCodec);
 
-    if (Subversion::Constants::debug)
-        qDebug() << "runSvn" << timeOut << outputText;
-
-    // Run, connect stderr to the output window
-    Utils::SynchronousProcess process;
-    if (!workingDir.isEmpty())
-        process.setWorkingDirectory(workingDir);
-    process.setTimeout(timeOut);
-    process.setStdOutCodec(outputCodec);
-
-    process.setStdErrBufferedSignalsEnabled(true);
-    connect(&process, SIGNAL(stdErrBuffered(QString,bool)), outputWindow, SLOT(append(QString)));
-
-    // connect stdout to the output window if desired
-    if (showStdOutInOutputWindow) {
-        process.setStdOutBufferedSignalsEnabled(true);
-        connect(&process, SIGNAL(stdOutBuffered(QString,bool)), outputWindow, SLOT(append(QString)));
-    }
-
-    const Utils::SynchronousProcessResponse sp_resp = process.run(executable, allArgs);
-    response.error = true;
+    response.error = sp_resp.result != Utils::SynchronousProcessResponse::Finished;
+    if (response.error)
+        response.message = sp_resp.exitMessage(executable, timeOut);
     response.stdErr = sp_resp.stdErr;
     response.stdOut = sp_resp.stdOut;
-    switch (sp_resp.result) {
-    case Utils::SynchronousProcessResponse::Finished:
-        response.error = false;
-        break;
-    case Utils::SynchronousProcessResponse::FinishedError:
-        response.message = tr("The process terminated with exit code %1.").arg(sp_resp.exitCode);
-        break;
-    case Utils::SynchronousProcessResponse::TerminatedAbnormally:
-        response.message = tr("The process terminated abnormally.");
-        break;
-    case Utils::SynchronousProcessResponse::StartFailed:
-        response.message = tr("Could not start subversion '%1'. Please check your settings in the preferences.").arg(executable);
-        break;
-    case Utils::SynchronousProcessResponse::Hang:
-        response.message = tr("Subversion did not respond within timeout limit (%1 ms).").arg(timeOut);
-        break;
-    }
-    if (response.error)
-        outputWindow->appendError(response.message);
-
     return response;
 }
 
@@ -1125,7 +1125,7 @@ Core::IEditor * SubversionPlugin::showOutputInEditor(const QString& title, const
     if (codec)
         e->setCodec(codec);
     Core::IEditor *ie = e->editableInterface();
-    Core::EditorManager::instance()->activateEditor(ie);
+    Core::EditorManager::instance()->activateEditor(ie, Core::EditorManager::ModeSwitch);
     return ie;
 }
 
@@ -1164,7 +1164,9 @@ bool SubversionPlugin::vcsAdd15(const QString &workingDir, const QString &rawFil
     const QString file = QDir::toNativeSeparators(rawFileName);
     QStringList args;
     args << QLatin1String("add") << QLatin1String("--parents") << file;
-    const SubversionResponse response = runSvn(workingDir, args, m_settings.timeOutMS(), true);
+    const SubversionResponse response =
+            runSvn(workingDir, args, m_settings.timeOutMS(),
+                   SshPasswordPrompt|ShowStdOutInLogWindow);
     return !response.error;
 }
 
@@ -1182,10 +1184,12 @@ bool SubversionPlugin::vcsAdd14(const QString &workingDir, const QString &rawFil
             if (!path.isEmpty())
                 path += slash;
             path += relativePath.at(p);
-            if (!managesDirectory(QDir(path))) {
+            if (!checkSVNSubDir(QDir(path))) {
                 QStringList addDirArgs;
                 addDirArgs << QLatin1String("add") << QLatin1String("--non-recursive") << QDir::toNativeSeparators(path);
-                const SubversionResponse addDirResponse = runSvn(workingDir, addDirArgs, m_settings.timeOutMS(), true);
+                const SubversionResponse addDirResponse =
+                        runSvn(workingDir, addDirArgs, m_settings.timeOutMS(),
+                               SshPasswordPrompt|ShowStdOutInLogWindow);
                 if (addDirResponse.error)
                     return false;
             }
@@ -1194,7 +1198,9 @@ bool SubversionPlugin::vcsAdd14(const QString &workingDir, const QString &rawFil
     // Add file
     QStringList args;
     args << QLatin1String("add") << QDir::toNativeSeparators(rawFileName);
-    const SubversionResponse response = runSvn(workingDir, args, m_settings.timeOutMS(), true);
+    const SubversionResponse response =
+            runSvn(workingDir, args, m_settings.timeOutMS(),
+                   SshPasswordPrompt|ShowStdOutInLogWindow);
     return !response.error;
 }
 
@@ -1205,23 +1211,130 @@ bool SubversionPlugin::vcsDelete(const QString &workingDir, const QString &rawFi
     QStringList args(QLatin1String("delete"));
     args.push_back(file);
 
-    const SubversionResponse response = runSvn(workingDir, args, m_settings.timeOutMS(), true);
+    const SubversionResponse response =
+            runSvn(workingDir, args, m_settings.timeOutMS(),
+                   SshPasswordPrompt|ShowStdOutInLogWindow);
     return !response.error;
 }
 
+bool SubversionPlugin::vcsMove(const QString &workingDir, const QString &from, const QString &to)
+{
+    QStringList args(QLatin1String("move"));
+    args << QDir::toNativeSeparators(from) << QDir::toNativeSeparators(to);
+    qDebug()<<args;
+    const SubversionResponse response =
+            runSvn(workingDir, args, m_settings.timeOutMS(),
+                   SshPasswordPrompt|ShowStdOutInLogWindow);
+    qDebug() << response.stdOut << "\n"<<response.stdErr;
+    return !response.error;
+}
+
+bool SubversionPlugin::vcsCheckout(const QString &directory, const QByteArray &url)
+{
+    QUrl tempUrl;
+    tempUrl.setEncodedUrl(url);
+    QString username = tempUrl.userName();
+    QString password = tempUrl.password();
+    QStringList args = QStringList(QLatin1String("checkout"));
+    args << QLatin1String(nonInteractiveOptionC) ;
+
+    if(!username.isEmpty() && !password.isEmpty())
+    {
+        // If url contains username and password we have to use separate username and password
+        // arguments instead of passing those in the url. Otherwise the subversion 'non-interactive'
+        // authentication will always fail (if the username and password data are not stored locally),
+        // if for example we are logging into a new host for the first time using svn. There seems to
+        // be a bug in subversion, so this might get fixed in the future.
+        tempUrl.setUserInfo("");
+        args << tempUrl.toEncoded() << directory;
+        const SubversionResponse response = runSvn(directory, username, password, args,
+                                                   m_settings.longTimeOutMS(),
+                                                   VCSBase::VCSBasePlugin::SshPasswordPrompt);
+        return !response.error;
+    } else {
+        args << url << directory;
+        const SubversionResponse response = runSvn(directory, args, m_settings.longTimeOutMS(),
+                                                   VCSBase::VCSBasePlugin::SshPasswordPrompt);
+        return !response.error;
+    }
+}
+
+QString SubversionPlugin::vcsGetRepositoryURL(const QString &directory)
+{
+    QXmlStreamReader xml;
+    QStringList args = QStringList(QLatin1String("info"));
+    args << QLatin1String("--xml");
+
+    const SubversionResponse response = runSvn(directory, args, m_settings.longTimeOutMS(), SuppressCommandLogging);
+    xml.addData(response.stdOut);
+
+    bool repo = false;
+    bool root = false;
+
+    while(!xml.atEnd() && !xml.hasError()) {
+        switch(xml.readNext()) {
+        case QXmlStreamReader::StartDocument:
+            break;
+        case QXmlStreamReader::StartElement:
+            if(xml.name() == QLatin1String("repository"))
+                repo = true;
+            else if(repo && xml.name() == QLatin1String("root"))
+                root = true;
+            break;
+        case QXmlStreamReader::EndElement:
+            if(xml.name() == QLatin1String("repository"))
+                repo = false;
+            else if(repo && xml.name() == QLatin1String("root"))
+                root = false;
+            break;
+        case QXmlStreamReader::Characters:
+            if (repo && root)
+                return xml.text().toString();
+            break;
+        default:
+            break;
+        }
+    }
+    return QString();
+}
+
 /* Subversion has ".svn" directory in each directory
  * it manages. The top level is the first directory
  * under the directory that does not have a  ".svn". */
-bool SubversionPlugin::managesDirectory(const QString &directory) const
+bool SubversionPlugin::managesDirectory(const QString &directory, QString *topLevel /* = 0 */) const
 {
     const QDir dir(directory);
-    const bool rc = dir.exists() && managesDirectory(dir);
-    if (Subversion::Constants::debug)
-        qDebug() << "SubversionPlugin::managesDirectory" << directory << rc;
-    return rc;
+    if (topLevel)
+        topLevel->clear();
+    bool manages = false;
+    do {
+        if (!dir.exists() || !checkSVNSubDir(dir))
+            break;
+        manages = true;
+        if (!topLevel)
+            break;
+        /* Recursing up, the top level is a child of the first directory that does
+         * not have a  ".svn" directory. The starting directory must be a managed
+         * one. Go up and try to find the first unmanaged parent dir. */
+        QDir lastDirectory = dir;
+        for (QDir parentDir = lastDirectory; parentDir.cdUp() ; lastDirectory = parentDir) {
+            if (!checkSVNSubDir(parentDir)) {
+                *topLevel = lastDirectory.absolutePath();
+                break;
+            }
+        }
+    } while (false);
+    if (Subversion::Constants::debug) {
+        QDebug nsp = qDebug().nospace();
+        nsp << "SubversionPlugin::managesDirectory" << directory << manages;
+        if (topLevel)
+            nsp << *topLevel;
+    }
+    return manages;
 }
 
-bool SubversionPlugin::managesDirectory(const QDir &directory) const
+// Check whether SVN management subdirs exist.
+bool SubversionPlugin::checkSVNSubDir(const QDir &directory) const
 {
     const int dirCount = m_svnDirectories.size();
     for (int i = 0; i < dirCount; i++) {
@@ -1232,30 +1345,6 @@ bool SubversionPlugin::managesDirectory(const QDir &directory) const
     return false;
 }
 
-QString SubversionPlugin::findTopLevelForDirectory(const QString &directory) const
-{
-    // Debug wrapper
-    const QString rc = findTopLevelForDirectoryI(directory);
-    if (Subversion::Constants::debug)
-        qDebug() << "SubversionPlugin::findTopLevelForDirectory" << directory << rc;
-    return rc;
-}
-
-QString SubversionPlugin::findTopLevelForDirectoryI(const QString &directory) const
-{
-    /* Recursing up, the top level is a child of the first directory that does
-     * not have a  ".svn" directory. The starting directory must be a managed
-     * one. Go up and try to find the first unmanaged parent dir. */
-    QDir lastDirectory = QDir(directory);
-    if (!lastDirectory.exists() || !managesDirectory(lastDirectory))
-        return QString();
-    for (QDir parentDir = lastDirectory; parentDir.cdUp() ; lastDirectory = parentDir) {
-        if (!managesDirectory(parentDir))
-            return lastDirectory.absolutePath();
-    }
-    return QString();
-}
-
 SubversionControl *SubversionPlugin::subVersionControl() const
 {
     return static_cast<SubversionControl *>(versionControl());
