@@ -3,6 +3,7 @@
 #undef private
 
 #include "gdb/gdbmi.h"
+#include "tcf/json.h"
 #include "gdbmacros.h"
 #include "gdbmacros_p.h"
 
@@ -14,6 +15,8 @@
 #include <QtTest/QtTest>
 //#include <QtTest/qtest_gui.h>
 
+//TESTED_COMPONENT=src/plugins/debugger/gdb
+
 static const char *pointerPrintFormat()
 {
     return sizeof(quintptr) == sizeof(long) ? "0x%lx" : "0x%llx";
@@ -95,6 +98,10 @@ static const char gdbmi12[] =
      "numchild=\"0\"}]";
 
 
+static const char jsont1[] =
+    "{\"Size\":100564,\"UID\":0,\"GID\":0,\"Permissions\":33261,"
+     "\"ATime\":1242370878000,\"MTime\":1239154689000}";
+
 struct Int3 {
     Int3() { i1 = 42; i2 = 43; i3 = 44; }
     int i1, i2, i3;
@@ -105,12 +112,12 @@ struct QString3 {
     QString s1, s2, s3;
 };
 
-class tst_Debugger : public QObject
+class tst_Dumpers : public QObject
 {
     Q_OBJECT
 
 public:
-    tst_Debugger() {}
+    tst_Dumpers() {}
 
     void testMi(const char* input)
     {
@@ -120,6 +127,12 @@ public:
             '\n' + QString(input));
     }
 
+    void testJson(const char* input)
+    {
+        QCOMPARE('\n' + QString::fromLatin1(JsonValue(input).toString(false)),
+            '\n' + QString(input));
+    }
+
 private slots:
     void mi1()  { testMi(gdbmi1);  }
     void mi2()  { testMi(gdbmi2);  }
@@ -132,6 +145,8 @@ private slots:
     void mi11() { testMi(gdbmi11); }
     //void mi12() { testMi(gdbmi12); }
 
+    void json1() { testJson(jsont1); }
+
     void infoBreak();
     void niceType();
     void niceType_data();
@@ -181,7 +196,6 @@ private slots:
 private:
     void dumpQAbstractItemHelper(QModelIndex &index);
     void dumpQAbstractItemModelHelper(QAbstractItemModel &m);
-    void dumpQDateTimeHelper(const QDateTime &d, bool isNull);
     template <typename K, typename V> void dumpQHashNodeHelper(QHash<K, V> &hash);
     void dumpQImageHelper(const QImage &img);
     void dumpQImageDataHelper(QImage &img);
@@ -200,7 +214,7 @@ private:
     void dumpQTextCodecHelper(QTextCodec *codec);
 };
 
-void tst_Debugger::infoBreak()
+void tst_Dumpers::infoBreak()
 {
     // This tests the regular expression used in GdbEngine::extractDataFromInfoBreak
     // to discover breakpoints in constructors.
@@ -345,7 +359,7 @@ QString niceType(QString type)
     return type;
 }
 
-void tst_Debugger::niceType()
+void tst_Dumpers::niceType()
 {
     // cf. watchutils.cpp
     QFETCH(QString, input);
@@ -353,7 +367,7 @@ void tst_Debugger::niceType()
     QCOMPARE(::niceType(input), simplified);
 }
 
-void tst_Debugger::niceType_data()
+void tst_Dumpers::niceType_data()
 {
     QTest::addColumn<QString>("input");
     QTest::addColumn<QString>("simplified");
@@ -437,7 +451,7 @@ static const void *deref(const void *p)
     return *reinterpret_cast<const char* const*>(p);
 }
 
-void tst_Debugger::dumperCompatibility()
+void tst_Dumpers::dumperCompatibility()
 {
     // Ensure that no arbitrary padding is introduced by QVectorTypedData.
     const size_t qVectorDataSize = 16;
@@ -605,7 +619,7 @@ void getMapNodeParams(size_t &nodeSize, size_t &valOffset)
 #endif
 }
 
-void tst_Debugger::dumpQAbstractItemHelper(QModelIndex &index)
+void tst_Dumpers::dumpQAbstractItemHelper(QModelIndex &index)
 {
     const QAbstractItemModel *model = index.model();
     const QString &rowStr = N(index.row());
@@ -644,7 +658,7 @@ void tst_Debugger::dumpQAbstractItemHelper(QModelIndex &index)
     testDumper(expected, &index, NS"QAbstractItem", true, indexSpecValue);
 }
 
-void tst_Debugger::dumpQAbstractItemAndModelIndex()
+void tst_Dumpers::dumpQAbstractItemAndModelIndex()
 {
     class PseudoTreeItemModel : public QAbstractItemModel
     {
@@ -792,7 +806,7 @@ void tst_Debugger::dumpQAbstractItemAndModelIndex()
         &index4, NS"QModelIndex", true);
 }
 
-void tst_Debugger::dumpQAbstractItemModelHelper(QAbstractItemModel &m)
+void tst_Dumpers::dumpQAbstractItemModelHelper(QAbstractItemModel &m)
 {
     QByteArray address = ptrToBa(&m);
     QByteArray expected = QByteArray("tiname='iname',addr='%',"
@@ -826,7 +840,7 @@ void tst_Debugger::dumpQAbstractItemModelHelper(QAbstractItemModel &m)
     testDumper(expected, &m, NS"QAbstractItemModel", true);
 }
 
-void tst_Debugger::dumpQAbstractItemModel()
+void tst_Dumpers::dumpQAbstractItemModel()
 {
     // Case 1: No rows, one column.
     QStringList strList;
@@ -860,7 +874,7 @@ void tst_Debugger::dumpQAbstractItemModel()
     dumpQAbstractItemModelHelper(model);
 }
 
-void tst_Debugger::dumpQByteArray()
+void tst_Dumpers::dumpQByteArray()
 {
     // Case 1: Empty object.
     QByteArray ba;
@@ -906,7 +920,7 @@ void tst_Debugger::dumpQByteArray()
         &ba, NS"QByteArray", true);
 }
 
-void tst_Debugger::dumpQChar()
+void tst_Dumpers::dumpQChar()
 {
     // Case 1: Printable ASCII character.
     QChar c('X');
@@ -934,44 +948,33 @@ void tst_Debugger::dumpQChar()
         &c, NS"QChar", false);
 }
 
-void tst_Debugger::dumpQDateTimeHelper(const QDateTime &d, bool isNull)
+void tst_Dumpers::dumpQDateTime()
 {
-    QByteArray value;
-    if (d.isNull())
-        value = "value='(null)'";
-    else
-        value = QByteArray("value='%',valueencoded='2'")
-            << utfToBase64(d.toString());
+    // Case 1: Null object.
+    QDateTime d;
+    testDumper("value='(null)',type='$T',numchild='0'",
+        &d, NS"QDateTime", true);
 
-    QByteArray expected = QByteArray("%,type='$T',numchild='3',children=["
-        "{name='isNull',%},"
+    // Case 2: Non-null object.
+    d = QDateTime::currentDateTime();
+    testDumper(QByteArray("value='%',valueencoded='2',"
+        "type='$T',numchild='1',children=["
         "{name='toTime_t',%},"
         "{name='toString',%},"
         "{name='toString_(ISO)',%},"
         "{name='toString_(SystemLocale)',%},"
         "{name='toString_(Locale)',%}]")
-            << value
-            << generateBoolSpec(d.isNull())
+            << utfToBase64(d.toString())
             << generateLongSpec((d.toTime_t()))
-            << generateQStringSpec(d.toString(), isNull)
-            << generateQStringSpec(d.toString(Qt::ISODate), isNull)
-            << generateQStringSpec(d.toString(Qt::SystemLocaleDate), isNull)
-            << generateQStringSpec(d.toString(Qt::LocaleDate), isNull);
-    testDumper(expected, &d, NS"QDateTime", true);
+            << generateQStringSpec(d.toString())
+            << generateQStringSpec(d.toString(Qt::ISODate))
+            << generateQStringSpec(d.toString(Qt::SystemLocaleDate))
+            << generateQStringSpec(d.toString(Qt::LocaleDate)),
+        &d, NS"QDateTime", true);
+
 }
 
-void tst_Debugger::dumpQDateTime()
-{
-    // Case 1: Null object.
-    QDateTime d;
-    dumpQDateTimeHelper(d, true);
-
-    // Case 2: Non-null object.
-    d = QDateTime::currentDateTime();
-    dumpQDateTimeHelper(d, false);
-}
-
-void tst_Debugger::dumpQDir()
+void tst_Dumpers::dumpQDir()
 {
     // Case 1: Current working directory.
     QDir dir = QDir::current();
@@ -993,7 +996,7 @@ void tst_Debugger::dumpQDir()
 }
 
 
-void tst_Debugger::dumpQFile()
+void tst_Dumpers::dumpQFile()
 {
     // Case 1: Empty file name => Does not exist.
     QFile file1("");
@@ -1023,7 +1026,7 @@ void tst_Debugger::dumpQFile()
         &file3, NS"QFile", true);
 }
 
-void tst_Debugger::dumpQFileInfo()
+void tst_Dumpers::dumpQFileInfo()
 {
     QFileInfo fi(".");
     QByteArray expected("value='%',valueencoded='2',type='$T',numchild='3',"
@@ -1125,7 +1128,7 @@ void tst_Debugger::dumpQFileInfo()
     testDumper(expected, &fi, NS"QFileInfo", true);
 }
 
-void tst_Debugger::dumpQHash()
+void tst_Dumpers::dumpQHash()
 {
     QHash<QString, QList<int> > hash;
     hash.insert("Hallo", QList<int>());
@@ -1135,7 +1138,7 @@ void tst_Debugger::dumpQHash()
 }
 
 template <typename K, typename V>
-void tst_Debugger::dumpQHashNodeHelper(QHash<K, V> &hash)
+void tst_Dumpers::dumpQHashNodeHelper(QHash<K, V> &hash)
 {
     typename QHash<K, V>::iterator it = hash.begin();
     typedef QHashNode<K, V> HashNode;
@@ -1156,7 +1159,7 @@ void tst_Debugger::dumpQHashNodeHelper(QHash<K, V> &hash)
         getMapType<K, V>(), "", sizeof(it.key()), sizeof(it.value()));
 }
 
-void tst_Debugger::dumpQHashNode()
+void tst_Dumpers::dumpQHashNode()
 {
     // Case 1: simple type -> simple type.
     QHash<int, int> hash1;
@@ -1179,17 +1182,15 @@ void tst_Debugger::dumpQHashNode()
     dumpQHashNodeHelper(hash4);
 }
 
-void tst_Debugger::dumpQImageHelper(const QImage &img)
+void tst_Dumpers::dumpQImageHelper(const QImage &img)
 {
-    QByteArray expected = "value='(%x%)',type='"NS"QImage',numchild='1',"
-        "children=[{name='data',type='"NS"QImageData',addr='%'}]"
+    QByteArray expected = "value='(%x%)',type='"NS"QImage',numchild='0'"
             << N(img.width())
-            << N(img.height())
-            << ptrToBa(&img);
+            << N(img.height());
     testDumper(expected, &img, NS"QImage", true);
 }
 
-void tst_Debugger::dumpQImage()
+void tst_Dumpers::dumpQImage()
 {
     // Case 1: Null image.
     QImage img;
@@ -1204,7 +1205,7 @@ void tst_Debugger::dumpQImage()
     dumpQImageHelper(img);
 }
 
-void tst_Debugger::dumpQImageDataHelper(QImage &img)
+void tst_Dumpers::dumpQImageDataHelper(QImage &img)
 {
     const QByteArray ba(QByteArray::fromRawData((const char*) img.bits(), img.numBytes()));
     QByteArray expected = QByteArray("tiname='$I',addr='$A',type='"NS"QImageData',").
@@ -1214,7 +1215,7 @@ void tst_Debugger::dumpQImageDataHelper(QImage &img)
     testDumper(expected, &img, NS"QImageData", false);
 }
 
-void tst_Debugger::dumpQImageData()
+void tst_Dumpers::dumpQImageData()
 {
     // Case 1: Null image.
     QImage img;
@@ -1230,7 +1231,7 @@ void tst_Debugger::dumpQImageData()
 }
 
 template <typename T>
-        void tst_Debugger::dumpQLinkedListHelper(QLinkedList<T> &l)
+        void tst_Dumpers::dumpQLinkedListHelper(QLinkedList<T> &l)
 {
     const int size = qMin(l.size(), 1000);
     const QString &sizeStr = N(size);
@@ -1268,7 +1269,7 @@ template <typename T>
     testDumper(expected, &l, NS"QLinkedList", true, elemTypeStr);
 }
 
-void tst_Debugger::dumpQLinkedList()
+void tst_Dumpers::dumpQLinkedList()
 {
     // Case 1: Simple element type.
     QLinkedList<int> l;
@@ -1310,7 +1311,7 @@ void tst_Debugger::dumpQLinkedList()
     dumpQLinkedListHelper(l3);
 }
 
-    #if 0
+#    if 0
     void tst_Debugger::dumpQLinkedList()
     {
         // Case 1: Simple element type.
@@ -1389,9 +1390,9 @@ void tst_Debugger::dumpQLinkedList()
         //dumpQLinkedListHelper(l3);
         testDumper("", &l, NS"QLinkedList", true, NS"QString");
     }
-    #endif
+#    endif
 
-void tst_Debugger::dumpQList_int()
+void tst_Dumpers::dumpQList_int()
 {
     QList<int> ilist;
     testDumper("value='<0 items>',valueeditable='false',numchild='0',"
@@ -1406,7 +1407,7 @@ void tst_Debugger::dumpQList_int()
         &ilist, NS"QList", true, "int");
 }
 
-void tst_Debugger::dumpQList_int_star()
+void tst_Dumpers::dumpQList_int_star()
 {
     QList<int *> ilist;
     testDumper("value='<0 items>',valueeditable='false',numchild='0',"
@@ -1422,7 +1423,7 @@ void tst_Debugger::dumpQList_int_star()
         &ilist, NS"QList", true, "int*");
 }
 
-void tst_Debugger::dumpQList_char()
+void tst_Dumpers::dumpQList_char()
 {
     QList<char> clist;
     testDumper("value='<0 items>',valueeditable='false',numchild='0',"
@@ -1437,7 +1438,7 @@ void tst_Debugger::dumpQList_char()
         &clist, NS"QList", true, "char");
 }
 
-void tst_Debugger::dumpQList_QString()
+void tst_Dumpers::dumpQList_QString()
 {
     QList<QString> slist;
     testDumper("value='<0 items>',valueeditable='false',numchild='0',"
@@ -1452,7 +1453,7 @@ void tst_Debugger::dumpQList_QString()
         &slist, NS"QList", true, NS"QString");
 }
 
-void tst_Debugger::dumpQList_Int3()
+void tst_Dumpers::dumpQList_Int3()
 {
     QList<Int3> i3list;
     testDumper("value='<0 items>',valueeditable='false',numchild='0',"
@@ -1467,7 +1468,7 @@ void tst_Debugger::dumpQList_Int3()
         &i3list, NS"QList", true, "Int3");
 }
 
-void tst_Debugger::dumpQList_QString3()
+void tst_Dumpers::dumpQList_QString3()
 {
     QList<QString3> s3list;
     testDumper("value='<0 items>',valueeditable='false',numchild='0',"
@@ -1482,7 +1483,7 @@ void tst_Debugger::dumpQList_QString3()
         &s3list, NS"QList", true, "QString3");
 }
 
-void tst_Debugger::dumpQLocaleHelper(QLocale &loc)
+void tst_Dumpers::dumpQLocaleHelper(QLocale &loc)
 {
     QByteArray expected = QByteArray("value='%',type='$T',numchild='8',"
             "children=[{name='country',%},"
@@ -1513,7 +1514,7 @@ void tst_Debugger::dumpQLocaleHelper(QLocale &loc)
     testDumper(expected, &loc, NS"QLocale", true);
 }
 
-void tst_Debugger::dumpQLocale()
+void tst_Dumpers::dumpQLocale()
 {
     QLocale english(QLocale::English);
     dumpQLocaleHelper(english);
@@ -1529,7 +1530,7 @@ void tst_Debugger::dumpQLocale()
 }
 
 template <typename K, typename V>
-        void tst_Debugger::dumpQMapHelper(QMap<K, V> &map)
+        void tst_Dumpers::dumpQMapHelper(QMap<K, V> &map)
 {
     QByteArray sizeStr(valToString(map.size()));
     size_t nodeSize;
@@ -1582,7 +1583,7 @@ template <typename K, typename V>
                true, getMapType<K,V>(), "", 0, 0, nodeSize, valOff);
 }
 
-void tst_Debugger::dumpQMap()
+void tst_Dumpers::dumpQMap()
 {
     qDebug() << "QMap<int, int>";
     // Case 1: Simple type -> simple type.
@@ -1649,7 +1650,7 @@ void tst_Debugger::dumpQMap()
 }
 
 template <typename K, typename V>
-        void tst_Debugger::dumpQMapNodeHelper(QMap<K, V> &m)
+        void tst_Dumpers::dumpQMapNodeHelper(QMap<K, V> &m)
 {
     typename QMap<K, V>::iterator it = m.begin();
     const K &key = it.key();
@@ -1669,7 +1670,7 @@ template <typename K, typename V>
                true, getMapType<K,V>(), "", 0, 0, nodeSize, valOffset);
 }
 
-void tst_Debugger::dumpQMapNode()
+void tst_Dumpers::dumpQMapNode()
 {
     // Case 1: simple type -> simple type.
     QMap<int, int> map;
@@ -1692,7 +1693,7 @@ void tst_Debugger::dumpQMapNode()
     dumpQMapNodeHelper(map4);
 }
 
-void tst_Debugger::dumpQObject()
+void tst_Dumpers::dumpQObject()
 {
     QObject parent;
     testDumper("value='',valueencoded='2',type='$T',displayedtype='QObject',"
@@ -1751,7 +1752,7 @@ void tst_Debugger::dumpQObject()
         &child, NS"QObject", false);
 }
 
-void tst_Debugger::dumpQObjectChildListHelper(QObject &o)
+void tst_Dumpers::dumpQObjectChildListHelper(QObject &o)
 {
     const QObjectList children = o.children();
     const int size = children.size();
@@ -1771,7 +1772,7 @@ void tst_Debugger::dumpQObjectChildListHelper(QObject &o)
     testDumper(expected, &o, NS"QObjectChildList", true);
 }
 
-void tst_Debugger::dumpQObjectChildList()
+void tst_Dumpers::dumpQObjectChildList()
 {
     // Case 1: Object with no children.
     QObject o;
@@ -1786,7 +1787,7 @@ void tst_Debugger::dumpQObjectChildList()
     dumpQObjectChildListHelper(o);
 }
 
-void tst_Debugger::dumpQObjectMethodList()
+void tst_Dumpers::dumpQObjectMethodList()
 {
     QStringListModel m;
     testDumper("addr='<synthetic>',type='$T',numchild='24',"
@@ -1818,7 +1819,7 @@ void tst_Debugger::dumpQObjectMethodList()
     &m, NS"QObjectMethodList", true);
 }
 
-void tst_Debugger::dumpQObjectPropertyList()
+void tst_Dumpers::dumpQObjectPropertyList()
 {
     // Case 1: Model without a parent.
     QStringListModel m(QStringList() << "Test1" << "Test2");
@@ -1871,7 +1872,7 @@ public:
 
 typedef QVector<QObjectPrivate::ConnectionList> ConnLists;
 
-void tst_Debugger::dumpQObjectSignalHelper(QObject &o, int sigNum)
+void tst_Dumpers::dumpQObjectSignalHelper(QObject &o, int sigNum)
 {
     //qDebug() << o.objectName() << sigNum;
     QByteArray expected("addr='<synthetic>',numchild='1',type='"NS"QObjectSignal'");
@@ -1916,7 +1917,7 @@ void tst_Debugger::dumpQObjectSignalHelper(QObject &o, int sigNum)
     testDumper(expected, &o, NS"QObjectSignal", true, "", "", sigNum);
 }
 
-void tst_Debugger::dumpQObjectSignal()
+void tst_Dumpers::dumpQObjectSignal()
 {
     // Case 1: Simple QObject.
     QObject o;
@@ -1951,7 +1952,7 @@ void tst_Debugger::dumpQObjectSignal()
         dumpQObjectSignalHelper(m, signalIndex);
 }
 
-void tst_Debugger::dumpQObjectSignalList()
+void tst_Dumpers::dumpQObjectSignalList()
 {
     // Case 1: Simple QObject.
     QObject o;
@@ -2050,7 +2051,7 @@ QByteArray slotIndexList(const QObject *ob)
     return slotIndices;
 }
 
-void tst_Debugger::dumpQObjectSlot()
+void tst_Dumpers::dumpQObjectSlot()
 {
     // Case 1: Simple QObject.
     QObject o;
@@ -2106,7 +2107,7 @@ void tst_Debugger::dumpQObjectSlot()
 
 }
 
-void tst_Debugger::dumpQObjectSlotList()
+void tst_Dumpers::dumpQObjectSlotList()
 {
     // Case 1: Simple QObject.
     QObject o;
@@ -2157,7 +2158,7 @@ void tst_Debugger::dumpQObjectSlotList()
         &m, NS"QObjectSlotList", true);
 }
 
-void tst_Debugger::dumpQPixmap()
+void tst_Dumpers::dumpQPixmap()
 {
     // Case 1: Null Pixmap.
     QPixmap p;
@@ -2185,7 +2186,7 @@ void tst_Debugger::dumpQPixmap()
 
 #if QT_VERSION >= 0x040500
 template<typename T>
-void tst_Debugger::dumpQSharedPointerHelper(QSharedPointer<T> &ptr)
+void tst_Dumpers::dumpQSharedPointerHelper(QSharedPointer<T> &ptr)
 {
     struct Cheater : public QSharedPointer<T>
     {
@@ -2224,7 +2225,7 @@ void tst_Debugger::dumpQSharedPointerHelper(QSharedPointer<T> &ptr)
 }
 #endif
 
-void tst_Debugger::dumpQSharedPointer()
+void tst_Dumpers::dumpQSharedPointer()
 {
 #if QT_VERSION >= 0x040500
     // Case 1: Simple type.
@@ -2264,7 +2265,7 @@ void tst_Debugger::dumpQSharedPointer()
 #endif
 }
 
-void tst_Debugger::dumpQString()
+void tst_Dumpers::dumpQString()
 {
     QString s;
     testDumper("value='IiIgKG51bGwp',valueencoded='5',type='$T',numchild='0'",
@@ -2274,14 +2275,14 @@ void tst_Debugger::dumpQString()
         &s, NS"QString", false);
 }
 
-void tst_Debugger::dumpQVariant_invalid()
+void tst_Dumpers::dumpQVariant_invalid()
 {
     QVariant v;
     testDumper("value='(invalid)',type='$T',numchild='0'",
         &v, NS"QVariant", false);
 }
 
-void tst_Debugger::dumpQVariant_QString()
+void tst_Dumpers::dumpQVariant_QString()
 {
     QVariant v = "abc";
     testDumper("value='KFFTdHJpbmcpICJhYmMi',valueencoded='5',type='$T',"
@@ -2296,7 +2297,7 @@ void tst_Debugger::dumpQVariant_QString()
 */
 }
 
-void tst_Debugger::dumpQVariant_QStringList()
+void tst_Dumpers::dumpQVariant_QStringList()
 {
     QVariant v = QStringList() << "Hi";
     testDumper("value='(QStringList) ',type='$T',numchild='1',"
@@ -2306,7 +2307,7 @@ void tst_Debugger::dumpQVariant_QStringList()
         &v, NS"QVariant", true);
 }
 
-void tst_Debugger::dumpStdVector()
+void tst_Dumpers::dumpStdVector()
 {
     std::vector<std::list<int> *> vector;
     QByteArray inner = "std::list<int> *";
@@ -2331,7 +2332,7 @@ void tst_Debugger::dumpStdVector()
     vector.push_back(0);
 }
 
-void tst_Debugger::dumpQTextCodecHelper(QTextCodec *codec)
+void tst_Dumpers::dumpQTextCodecHelper(QTextCodec *codec)
 {
     const QByteArray name = codec->name().toBase64();
     QByteArray expected = QByteArray("value='%',valueencoded='1',type='$T',"
@@ -2341,7 +2342,7 @@ void tst_Debugger::dumpQTextCodecHelper(QTextCodec *codec)
     testDumper(expected, codec, NS"QTextCodec", true);
 }
 
-void tst_Debugger::dumpQTextCodec()
+void tst_Dumpers::dumpQTextCodec()
 {
     const QList<QByteArray> &codecNames = QTextCodec::availableCodecs();
     foreach (const QByteArray &codecName, codecNames)
@@ -2357,7 +2358,7 @@ template <typename T1, typename T2>
 }
 
 template <typename T>
-void tst_Debugger::dumpQWeakPointerHelper(QWeakPointer<T> &ptr)
+void tst_Dumpers::dumpQWeakPointerHelper(QWeakPointer<T> &ptr)
 {
     typedef QtSharedPointer::ExternalRefCountData Data;
     const size_t dataOffset = 0;
@@ -2381,7 +2382,7 @@ void tst_Debugger::dumpQWeakPointerHelper(QWeakPointer<T> &ptr)
 }
 #endif
 
-void tst_Debugger::dumpQWeakPointer()
+void tst_Dumpers::dumpQWeakPointer()
 {
 #if QT_VERSION >= 0x040500
     // Case 1: Simple type.
@@ -2420,7 +2421,7 @@ do {                                                      \
 } while (0)
 
 
-void tst_Debugger::initTestCase()
+void tst_Dumpers::initTestCase()
 {
     QVERIFY(sizeof(QWeakPointer<int>) == 2*sizeof(void *));
     QVERIFY(sizeof(QSharedPointer<int>) == 2*sizeof(void *));
@@ -2455,7 +2456,7 @@ void tst_Debugger::initTestCase()
 int main(int argc, char *argv[])
 {
     QApplication app(argc, argv);
-    tst_Debugger test;
+    tst_Dumpers test;
     return QTest::qExec(&test, argc, argv);
 }
 
